import {
  Component,
  HostListener,
  OnInit,
  signal,
  WritableSignal
} from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { GalleryService } from '../../services/gallery.service';
import { GalleryType } from '../../models/gallery.type';
import { NgClass, NgForOf, NgIf } from '@angular/common';
import { CascadeSelectModule } from 'primeng/cascadeselect';
import { PanelModule } from 'primeng/panel';
import {FileSelectEvent, FileUpload, FileUploadEvent, FileUploadModule} from 'primeng/fileupload';
import { ButtonGroupModule } from 'primeng/buttongroup';
import { catchError } from 'rxjs';
import {TabMenuModule} from 'primeng/tabmenu';
import {MenuItem} from 'primeng/api';

interface PayloadMenuItem extends MenuItem {

  payload: string

}

@Component({
  selector: 'app-gallery',
  standalone: true,
  imports: [
    NgIf,
    NgForOf,
    CascadeSelectModule,
    PanelModule,
    FileUploadModule,
    ButtonGroupModule,
    TabMenuModule
  ],
  templateUrl: './gallery.component.html',
  styleUrls: ['./gallery.component.css']
})
export class GalleryComponent implements OnInit {

  sorts: Array<PayloadMenuItem> = [
    {
      label: 'Najnowsze',
      payload: 'date'
    },
    {
      label: 'Popularne',
      payload: 'likes'
    }
  ];

  activeSort: PayloadMenuItem = this.sorts[0];

  gallery: GalleryType = { images: [] };
  selectedFile: File | null = null;
  previewUrl: string | ArrayBuffer | null = null;
  weddingId: string = '';


  constructor(private route: ActivatedRoute,
              private galleryService: GalleryService) {}

  ngOnInit(): void {
    this.weddingId = this.route.snapshot.paramMap.get('id') || '';
    if (this.weddingId) {
      this.loadGallery();
    } else {
      console.error('weddingId error');
    }
  }

  onImageDownloadAction(imageId: string): void {
    const url = `http://localhost:8080/wedding/${this.weddingId}/gallery/${imageId}`;
    const link = document.createElement('a');
    link.href = url;
    link.download = `${imageId}.jpg`;
    link.style.display = 'none';

    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
  }

  onUploadAction(event: FileSelectEvent): void {
    console.log(event);
    this.selectedFile = event.currentFiles[0];
    this.previewUrl = URL.createObjectURL(this.selectedFile);
  }

  onImageAddAction(): void {
    if (this.selectedFile) {
      this.galleryService.postImage(this.selectedFile, this.weddingId).subscribe({
        next: () => {
          this.onImageClearAction();
          this.loadGallery();
        },
        error: (err) => console.error('Error uploading image:', err)
      });
    }
  }

  onImageClearAction(): void {
    this.selectedFile = null;
    this.previewUrl = null;
  }

  onSortTypeChanged(sort: MenuItem): void {
    this.activeSort = sort as PayloadMenuItem;
    this.loadGallery();
  }

  onImageLikeAction(id: string): void {
    const likedImages: string[] = JSON.parse(localStorage.getItem('likedImages') || '[]');
    const index = likedImages.indexOf(id);

    if (index === -1) {
      likedImages.push(id);
      this.updateLikes(id, 1);
    } else {
      likedImages.splice(index, 1);
      this.updateLikes(id, -1);
    }

    localStorage.setItem('likedImages', JSON.stringify(likedImages));
  }

  updateLikes(id: string, delta: number): void {
    const image = this.gallery.images.find((img) => img.id === id);
    if (image) {
      // No, no, no. No delta. Send request of like and calculations done on backend.
      image.likes += delta;
      this.galleryService.changeImageLikes(this.weddingId, id, delta).subscribe({
        next: (response) => console.log('Like updated successfully:', response),
        error: (error) => console.error('Error updating like:', error)
      });
    }
  }

  loadGallery(): void {
    this.galleryService
      .getGalleryByWeddingId(this.weddingId, this.activeSort.payload)
      .pipe(catchError((error) => {
        console.error(error);
        throw error;
      }))
      .subscribe((weddingGallery) => (this.gallery = weddingGallery));
  }

}
